@@ -4,10 +4,10 @@
      https://github.com/walles/px
 
 Usage:
-  px
-  px <filter>
-  px <PID>
-  px --top
+  px [--debug]
+  px [--debug] <filter>
+  px [--debug] <PID>
+  px [--debug] --top
   px --install
   px --help
   px --version
@@ -34,15 +34,27 @@ of which processes are most active right now.
 --version: Print version information
 """
 
+import platform
+import logging
+import six
 import sys
-
 import os
+
 from . import px_install
 from . import px_process
 from . import px_terminal
 from . import px_processinfo
 
 
+ERROR_REPORTING_HEADER = """
+---
+
+Problems detected, please send this text to one of:
+* https://github.com/walles/px/issues/new
+* johan.walles@gmail.com
+"""
+
+
 def install(argv):
     # Find full path to self
     if not argv:
@@ -63,7 +75,49 @@ def main():
     _main(sys.argv)
 
 
+def configureLogging(loglevel, stringIO):
+    # type: (int, six.StringIO) -> None
+
+    # This method inspired by: https://stackoverflow.com/a/9534960/473672
+
+    rootLogger = logging.getLogger()
+    rootLogger.setLevel(loglevel)
+
+    handlers = []
+    for handler in rootLogger.handlers:
+        handlers.append(handler)
+    for handler in handlers:
+        rootLogger.removeHandler(handler)
+
+    handler = logging.StreamHandler(stringIO)
+    formatter = logging.Formatter(fmt='%(asctime)s %(levelname)-8s %(message)s',
+                                  datefmt='%Y-%m-%d %H:%M:%S%Z')
+    handler.setFormatter(formatter)
+
+    rootLogger.addHandler(handler)
+
+
+def handleLogMessages(messages):
+    if not messages:
+        return
+
+    sys.stderr.write(ERROR_REPORTING_HEADER)
+    sys.stderr.write("\n")
+    sys.stderr.write("Python version: " + sys.version + "\n")
+    sys.stderr.write("\n")
+    sys.stderr.write("Platform info: " + platform.platform() + "\n")
+    sys.stderr.write("\n")
+    sys.stderr.write(messages)
+    sys.stderr.write("\n")
+    sys.exit(1)
+
+
 def _main(argv):
+    loglevel = logging.ERROR
+    while '--debug' in argv:
+        argv.remove('--debug')
+        loglevel = logging.DEBUG
+
     if len(argv) == 1 and os.path.basename(argv[0]).endswith("top"):
         argv.append("--top")
 
@@ -82,10 +136,14 @@ def _main(argv):
         install(argv)
         return
 
+    stringIO = six.StringIO()
+    configureLogging(loglevel, stringIO)
+
     if arg == '--top':
         # Pulling px_top in on demand like this improves test result caching
         from . import px_top
         px_top.top()
+        handleLogMessages(stringIO.getvalue())
         return
 
     if arg == '--help':
@@ -105,7 +163,8 @@ def _main(argv):
 
     try:
         pid = int(arg)
-        px_processinfo.print_pid_info(pid)
+        px_processinfo.print_pid_info(sys.stdout.fileno(), pid)
+        handleLogMessages(stringIO.getvalue())
         return
     except ValueError:
         # It's a search filter and not a PID, keep moving
@@ -119,7 +178,7 @@ def _main(argv):
     window_size = px_terminal.get_window_size()
     if window_size is not None:
         columns = window_size[1]
-    lines = px_terminal.to_screen_lines(px_process.order_best_last(procs), columns)
+    lines = px_terminal.to_screen_lines(px_process.order_best_last(procs), columns, None, None)
     print("\n".join(lines))
 
 
