@@ -1,17 +1,17 @@
 # coding=utf-8
 
 import sys
-import tty
 import copy
 import time
 import errno
 import signal
 import select
-import termios
+import logging
 import unicodedata
 
 import os
 from . import px_load
+from . import px_pager
 from . import px_process
 from . import px_terminal
 from . import px_load_bar
@@ -19,14 +19,17 @@ from . import px_cpuinfo
 from . import px_processinfo
 from . import px_launchcounter
 
-if sys.version_info.major >= 3:
+if False:
     # For mypy PEP-484 static typing validation
+    import logging               # NOQA
     from typing import List      # NOQA
     from typing import Dict      # NOQA
     from typing import Union     # NOQA
     from typing import Optional  # NOQA
     from six import text_type    # NOQA
 
+LOG = logging.getLogger(__name__)
+
 # Used for informing our getch() function that a window resize has occured
 SIGWINCH_PIPE = os.pipe()
 
@@ -47,16 +50,15 @@ KEY_UPARROW = "\x1b[A"
 KEY_DOWNARROW = "\x1b[B"
 KEY_ENTER = "\x0d"
 
-SEARCH_PROMPT = px_terminal.bold("Search (ESC to cancel): ")
+SEARCH_PROMPT_ACTIVE = px_terminal.bold("Search (ENTER when done): ")
+SEARCH_PROMPT_INACTIVE = "Search ('/' to edit): "
 SEARCH_CURSOR = px_terminal.inverse_video(" ")
 
 MODE_BASE = 0
 MODE_SEARCH = 1
 
-initial_termios_attr = None  # type: Optional[List[Union[int, List[bytes]]]]
-
 top_mode = MODE_BASE  # type: int
-search_string = None  # type: Optional[text_type]
+search_string = ""  # type: text_type
 
 # Which pid were we last hovering?
 last_highlighted_pid = None  # type: Optional[int]
@@ -72,6 +74,9 @@ highlight_has_moved = False  # type: bool
 # When we last polled the system for a process list, in seconds since the Epoch
 last_process_poll = 0.0
 
+# Order top list by memory usage. The opposite is by CPU usage.
+sort_by_memory = False
+
 
 class ConsumableString(object):
     def __init__(self, string):
@@ -190,19 +195,35 @@ def getch(timeout_seconds=0, fd=None):
 
 
 def get_notnone_cpu_time_seconds(proc):
+    # type: (px_process.PxProcess) -> int
     seconds = proc.cpu_time_seconds
     if seconds is not None:
         return seconds
     return 0
 
 
-def get_toplist(baseline, current):
-    # type: (List[px_process.PxProcess], List[px_process.PxProcess]) -> List[px_process.PxProcess]
+def get_notnone_memory_percent(proc):
+    # type: (px_process.PxProcess) -> int
+    seconds = proc.memory_percent
+    if seconds is not None:
+        return seconds
+    return 0
+
+
+def get_toplist(baseline,  # type: List[px_process.PxProcess]
+                current,   # type: List[px_process.PxProcess]
+                by_memory=False  # type: bool
+                ):
+    # type(...) -> List[px_process.PxProcess]
     toplist = adjust_cpu_times(baseline, current)
 
     # Sort by CPU time used, then most interesting first
     toplist = px_process.order_best_first(toplist)
-    toplist = sorted(toplist, key=get_notnone_cpu_time_seconds, reverse=True)
+    if by_memory:
+        toplist = sorted(toplist, key=get_notnone_memory_percent, reverse=True)
+    else:
+        # By CPU time, this is the default
+        toplist = sorted(toplist, key=get_notnone_cpu_time_seconds, reverse=True)
 
     return toplist
 
@@ -212,23 +233,6 @@ def writebytes(bytestring):
     os.write(sys.stdout.fileno(), bytestring)
 
 
-def clear_screen():
-    """
-    Clear the screen and move cursor to top left corner:
-    https://en.wikipedia.org/wiki/ANSI_escape_code
-
-    Note that some experimentation was involved in coming up with this
-    exact sequence; if you do first "clear the full screen" then "home"
-    for example, the contents of the previous screen gets added to the
-    scrollback buffer, which isn't what we want. Tread carefully if you
-    intend to change these.
-    """
-
-    CSI = b"\x1b["
-    writebytes(CSI + b"1J")
-    writebytes(CSI + b"H")
-
-
 def get_line_to_highlight(toplist, max_process_count):
     # type: (List[px_process.PxProcess], int) -> Optional[int]
     global last_highlighted_pid
@@ -324,42 +328,36 @@ def get_screen_lines(
 
     # -2 = Section name + column headings
     max_process_count = cputop_height - 2
-    if search is not None:
-        # Search prompt needs one line
-        max_process_count -= 1
 
-    toplist_table_lines = px_terminal.to_screen_lines(toplist, columns)
-    if toplist_table_lines:
-        heading_line = toplist_table_lines[0]
-        heading_line = px_terminal.get_string_of_length(heading_line, columns)
-        heading_line = px_terminal.underline_bold(heading_line)
+    # Search prompt needs one line
+    max_process_count -= 1
 
-        highlight_me = get_line_to_highlight(toplist, max_process_count)
-        if highlight_me is not None:
-            # The "+ 1" here is to skip the heading line
-            highlighted = toplist_table_lines[highlight_me + 1]
-            highlighted = px_terminal.get_string_of_length(highlighted, columns)
-            highlighted = px_terminal.inverse_video(highlighted)
-
-            # The "+ 1" here is to skip the heading line
-            toplist_table_lines[highlight_me + 1] = highlighted
-
-        toplist_table_lines[0] = heading_line
+    highlight_row = get_line_to_highlight(toplist, max_process_count)
+    highlight_column = u"CPUTIME"
+    if sort_by_memory:
+        highlight_column = u"RAM"
+    toplist_table_lines = \
+        px_terminal.to_screen_lines(toplist, columns, highlight_row, highlight_column)
 
     # Ensure that we cover the whole screen, even if it's higher than the
     # number of processes
     toplist_table_lines += rows * ['']
 
     lines += [px_terminal.bold("Top CPU using processes")]
-    if search is not None:
-        lines += [SEARCH_PROMPT + search + SEARCH_CURSOR]
+    global top_mode
+    if top_mode == MODE_SEARCH:
+        lines += [SEARCH_PROMPT_ACTIVE + px_terminal.bold(search or "") + SEARCH_CURSOR]
+    else:
+        assert top_mode == MODE_BASE
+        lines += [SEARCH_PROMPT_INACTIVE + px_terminal.bold(search or "")]
 
     lines += toplist_table_lines[0:max_process_count + 1]  # +1 for the column headings
 
     lines += launchlines
 
     if include_footer:
-        footer_line = u"  q - Quit  / - Search  ↑↓ - Move selection  Enter - Select"
+        footer_line = \
+            u"  q - Quit  m - Sort order  / - Search  ↑↓ - Move  Enter - Select"
         footer_line = px_terminal.get_string_of_length(footer_line, columns)
         footer_line = px_terminal.inverse_video(footer_line)
 
@@ -387,28 +385,16 @@ def redraw(
     lines = get_screen_lines(
         load_bar, toplist, launchcounter, rows, columns, include_footer,
         search=search_string)
+
+    clear_sequence = u""
     if clear:
-        clear_screen()
+        clear_sequence = px_terminal.CLEAR_SCREEN
 
     # We need both \r and \n when the TTY is in tty.setraw() mode
-    writebytes(u"\r\n".join(lines).encode('utf-8'))
-    sys.stdout.flush()
+    writebytes((clear_sequence + u"\r\n".join(lines)).encode('utf-8'))
 
 
-def restore_display():
-    fd = sys.stdin.fileno()
-    tty.setcbreak(fd)
-
-    # tty.setraw() disables local echo, so we have to re-enable it here.
-    # See the "source code" comment to this answer:
-    # http://stackoverflow.com/a/8758047/473672
-    global initial_termios_attr
-    if initial_termios_attr:
-        termios.tcsetattr(fd, termios.TCSADRAIN, initial_termios_attr)
-
-
-# Print info about PID and exit
-def print_info_and_quit(pid):
+def page_process_info(pid):
     # type: (Optional[int]) -> None
     if pid is None:
         # Nothing selected, never mind
@@ -421,14 +407,8 @@ def print_info_and_quit(pid):
         # Process not available, never mind
         return
 
-    restore_display()
-
-    # Visually distance ourselves from the ptop view
-    print("")
-    print("")
-
-    px_processinfo.print_process_info(process, processes)
-    sys.exit(0)
+    with px_terminal.normal_display():
+        px_pager.page_process_info(process, processes)
 
 
 def handle_search_keypresses(key_sequence):
@@ -455,13 +435,10 @@ def handle_search_keypresses(key_sequence):
         elif key_sequence.consume(KEY_DOWNARROW):
             last_highlighted_row += 1
             last_highlighted_pid = None
-        elif key_sequence.consume(KEY_ENTER):
-            print_info_and_quit(last_highlighted_pid)
-        elif key_sequence._string == KEY_ESC:
+        elif key_sequence.consume(KEY_ENTER) or key_sequence._string == KEY_ESC:
             # Exit search mode
             global top_mode
             top_mode = MODE_BASE
-            search_string = None
             return
         else:
             # Unable to consume more, give up
@@ -503,6 +480,7 @@ def get_command(**kwargs):
 
     global last_highlighted_row
     global last_highlighted_pid
+    global sort_by_memory
     while len(input) > 0:
         if input.consume(KEY_UPARROW):
             last_highlighted_row -= 1
@@ -511,12 +489,13 @@ def get_command(**kwargs):
             last_highlighted_row += 1
             last_highlighted_pid = None
         elif input.consume(KEY_ENTER):
-            print_info_and_quit(last_highlighted_pid)
+            page_process_info(last_highlighted_pid)
         elif input.consume(u'/'):
             global search_string
             top_mode = MODE_SEARCH
-            search_string = ""
             return None
+        elif input.consume(u'm') or input.consume(u'M'):
+            sort_by_memory = not sort_by_memory
         elif input.consume(u'q'):
             return CMD_QUIT
         elif input.consume(SIGWINCH_KEY):
@@ -529,6 +508,8 @@ def get_command(**kwargs):
 
 
 def _top():
+    # type: () -> None
+
     physical, logical = px_cpuinfo.get_core_count()
     load_bar = px_load_bar.PxLoadBar(physical, logical)
     baseline = px_process.get_all()
@@ -541,7 +522,8 @@ def _top():
             sys.stderr.write("Cannot find terminal window size, are you on a terminal?\r\n")
             exit(1)
         rows, columns = window_size
-        toplist = get_toplist(baseline, current)
+        global sort_by_memory
+        toplist = get_toplist(baseline, current, sort_by_memory)
         redraw(load_bar, toplist, launchcounter, rows, columns)
 
         command = get_command(timeout_seconds=1)
@@ -575,23 +557,18 @@ def sigwinch_handler(signum, frame):
 
 
 def top():
+    # type: () -> None
+
     if not sys.stdout.isatty():
         sys.stderr.write('Top mode only works on TTYs, try running just "px" instead.\n')
         exit(1)
 
     signal.signal(signal.SIGWINCH, sigwinch_handler)
-
-    fd = sys.stdin.fileno()
-
-    global initial_termios_attr
-    initial_termios_attr = termios.tcgetattr(sys.stdin.fileno())
-
-    tty.setraw(fd)
-
-    try:
-        _top()
-    finally:
-        restore_display()
+    with px_terminal.fullscreen_display():
+        try:
+            _top()
+        except Exception:
+            LOG.exception("Running ptop failed")
 
         # Make sure we actually end up on a new line
         print("")
