@@ -162,7 +162,7 @@ def test_str_resolve():
     test_me.type = "IPv6"
     test_me.name = "[::1]:17600"
 
-    match = re.match("^\[IPv6\] (.*):17600 \(LISTEN\)$", str(test_me))
+    match = re.match(r"^\[IPv6\] (.*):17600 \(LISTEN\)$", str(test_me))
     assert match
     resolution = match.group(1)
     assert resolution == "[::1]" or "localhost" in resolution
