@@ -5,6 +5,9 @@ import dateutil.tz
 
 from px import px_loginhistory
 
+if False:
+    from typing import Set  # NOQA
+
 
 @pytest.yield_fixture
 def check_output(capfd):
@@ -16,6 +19,7 @@ def check_output(capfd):
 
 
 def get_users_at(last_output, now, testtime):
+    # type: (str, datetime.datetime, datetime.datetime) -> Set[str]
     """
     Ask px_loginhistory to parse last_output given the current timestamp of now.
 
@@ -254,16 +258,13 @@ def test_get_users_at_trailing_noise(check_output):
     assert not get_users_at("wtmp begins Thu Oct  1 22:54 ", now, now)
 
 
-def test_get_users_at_unexpected_last_output(capfd):
+def test_get_users_at_unexpected_last_output(caplog):
     UNEXPECTED = "glasskiosk"
 
     now = datetime.datetime(2016, 4, 7, 12, 8, tzinfo=dateutil.tz.tzlocal())
     assert not get_users_at(UNEXPECTED, now, now)
 
-    out, err = capfd.readouterr()
-    assert not out
-    assert UNEXPECTED in err
-    assert 'https://github.com/walles/px/issues' in err
+    assert UNEXPECTED in caplog.text
 
 
 def test_get_users_at_just_run_it(check_output):
